# This is sample code to load the AtomDB CX model into XSPEC
# Not all model parameters are implemented
# Filenames are explicitly defined due to beta status
#
# Requires installation of pyatomdb
# 2017-01-09 Adam Foster

import xspec, pyatomdb

ellist=[6,7,8,10,12,14,16,18,20,26,28]
acxH = pyatomdb.spectrum.CXSession(linefile = 'apec_v3.0.9_kronos_H_line.fits',\
                                 cocofile = 'apec_v3.0.9_kronos_H_comp.fits',\
                                 acxlinefile = 'apec_v3.0.9_acx_H_line.fits',\
                                 acxcocofile = 'apec_v3.0.9_acx_H_comp.fits',\
                                 elements = ellist, \
                                 abundset = 'AG89',\
                                 collisionunits = 'kev/amu',\
                                 veltype=1,\
                                 donor='H',\
                                 acxmodel = 8)

acxHe = pyatomdb.spectrum.CXSession(linefile = 'apec_v3.0.9_kronos_He_line.fits',\
                                 cocofile = 'apec_v3.0.9_kronos_He_comp.fits',\
                                 acxlinefile = 'apec_v3.0.9_acx_He_line.fits',\
                                 acxcocofile = 'apec_v3.0.9_acx_He_comp.fits',\
                                 elements = ellist, \
                                 abundset = 'AG89',\
                                 collisionunits = 'kev/amu',\
                                 veltype=1,\
                                 donor='He',\
                                 acxmodel = 8)

acxsetup=False
acxH.acxsetup=False
acxInfo = ("temperature   \"keV\"   1.0 0.00862 0.00862  86.  86.  0.01",
           "collenergy    \"kev/u\" 1.0 0.01 0.2 100. 1000. 0.01",
           "acxmodel      \"\"      8.0 1.0 1.0 16.0 16.0 -0.01",
           "Hefrac        \"\"      0.09 0.0 0.0 1.0 1.0 -0.01",
           "abund         \"\"      1.0 0.0 0.0 10.0 10.0 0.01")


def checkabundset():
  """
  Check the current abundance set matches that used in XSPEC. If not, adjust accordingly
  
  PARAMETERS
  ----------
  
  abundset_in : str
    The abundance set, as defined by XSPEC (so one of feld,angr,aneb,grsa,wilm,lodd,aspl)
  
  RETURNS
  -------
  none
  
  NOTES
  -----
  Updates the abundset vector in acxH and acxHe
  NOT OPERATIONAL YET UNLESS YOU HAVE INSTALLED CUSTOM ABUNDANCE FILE
  Stick with the XSPEC default for now (angr)
  """
  
  # grid of translations from XSPEC to ATOMDB names for abundances
  
  xs2at = {}
  xs2at['feld']='Feldman'
  xs2at['angr']='AG89'
  xs2at['aneb']='AE82'
  xs2at['grsa']='GS98'
  xs2at['wilm']='Wilm00'
  xs2at['lodd']='Lodd03'
  xs2at['aspl']='Asplund09'
  
  # get the xspec abundance set
  xabund = xspec.Xset.abund

  # change the abundance set if required.
  if acxH.abundset != xs2at[xabund]:
    acxH.set_abundset(xs2at[xabund])
  if acxHe.abundset != xs2at[xabund]:
    acxHe.set_abundset(xs2at[xabund])
    

def xspecacx(engs, params, flux):
  # initial run
  if not acxH.acxsetup:
    acxH.set_specbins(pyatomdb.numpy.array(engs))
    acxHe.set_specbins(pyatomdb.numpy.array(engs))
    acxH.set_ionbal_temperature(params[0])
    acxHe.set_ionbal_temperature(params[0])
    acxH.acxsetup = True

  # recalc ionization balance if needed    
  if acxH.ionbal_temperature != params[0]:
    acxH.set_ionbal_temperature(params[0])
    acxHe.set_ionbal_temperature(params[0])

  # set up the acxmodel number if it has changed
  if int(params[2]) != acxH.acxmodel:
    acxH.set_acxmodel(int(params[2]))
    acxHe.set_acxmodel(int(params[2]))

  # abundances
  checkabundset()

  elements = numpy.arange(1,31, dtype=int)
  abund = params[4]
  
  acxH.set_abund(elements, abund)
  acxHe.set_abund(elements, abund)

  # calculate spectra    
  Hspec = acxH.return_spectra(params[1])
  Hespec = acxHe.return_spectra(params[1])

  
  # Helium fraction   
  tmpflux = Hespec*params[3] + Hspec*(1-params[3])

  # return flux
  flux[:]=tmpflux

# add acx as a possible XSPEC model
xspec.AllModels.addPyMod(xspecacx, acxInfo, 'add')


# Here, actually declare this as the model you want to use
# presumably, most of the time this would not be a stand alone model
# i.e. something like xspec.Model('tbabs(powerlaw+xspecacx)')
xspec.Model('xspecacx')

# ...then proceed with loading data, fitting, etc.






# this is a similar thing but effectively a vvacx - 
# you can adjust the abundances of each element individually



# for speed, constrain the elements. In theory, this is unnecessary
# as all the elements are included.

ellist=[6,7,8,10,12,14,16,18,20,26,28]
acxH = pyatomdb.spectrum.CXSession(linefile = 'apec_v3.0.9_kronos_H_line.fits',\
                                 cocofile = 'apec_v3.0.9_kronos_H_comp.fits',\
                                 acxlinefile = 'apec_v3.0.9_acx_H_line.fits',\
                                 acxcocofile = 'apec_v3.0.9_acx_H_comp.fits',\
                                 elements = ellist, \
                                 abundset = 'AG89',\
                                 collisionunits = 'kev/amu',\
                                 veltype=1,\
                                 donor='H',\
                                 acxmodel = 8)

acxHe = pyatomdb.spectrum.CXSession(linefile = 'apec_v3.0.9_kronos_He_line.fits',\
                                 cocofile = 'apec_v3.0.9_kronos_He_comp.fits',\
                                 acxlinefile = 'apec_v3.0.9_acx_He_line.fits',\
                                 acxcocofile = 'apec_v3.0.9_acx_He_comp.fits',\
                                 elements = ellist, \
                                 abundset = 'AG89',\
                                 collisionunits = 'kev/amu',\
                                 veltype=1,\
                                 donor='He',\
                                 acxmodel = 8)

acxsetup=False
acxH.acxsetup=False
vvacxInfo = ("temperature   \"keV\"   1.0 0.00862 0.00862  86.  86.  0.01",
           "collenergy    \"kev/u\" 1.0 0.01 0.2 100. 1000. 0.01",
           "acxmodel      \"\"      8.0 1.0 1.0 16.0 16.0 -0.01",
           "Hefrac        \"\"      0.09 0.0 0.0 1.0 1.0 -0.01",
           "H             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "He            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Li            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Be            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "B             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "C             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "N             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "O             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "F             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Ne            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Na            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Mg            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Al            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Si            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "P             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "S             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Cl            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Ar            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "K             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Ca            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Sc            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Ti            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "V             \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Cr            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Mn            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Fe            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Co            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Ni            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Cu            \"\"      1.0 0.0 0.0 10.0 10.0 0.01",
           "Zn            \"\"      1.0 0.0 0.0 10.0 10.0 0.01")



def xspecvvacx(engs, params, flux):
  # initial run
  if not acxH.acxsetup:
    acxH.set_specbins(pyatomdb.numpy.array(engs))
    acxHe.set_specbins(pyatomdb.numpy.array(engs))
    acxH.set_ionbal_temperature(params[0])
    acxHe.set_ionbal_temperature(params[0])
    
    acxH.acxsetup = True

  # recalc ionization balance if needed    
  if acxH.ionbal_temperature != params[0]:
    acxH.set_ionbal_temperature(params[0])
    acxHe.set_ionbal_temperature(params[0])
    
  # set up the acxmodel number if it has changed
  if int(params[2]) != acxH.acxmodel:
    acxH.set_acxmodel(int(params[2]))
    acxHe.set_acxmodel(int(params[2]))

  # abundances
  checkabundset()
  
  elements = numpy.arange(1,31, dtype=int)
  abund = numpy.array(params[4:34])
  acxH.set_abund(elements, abund)
  acxHe.set_abund(elements, abund)

  # calculate spectra    
  Hspec = acxH.return_spectra(params[1])
  Hespec = acxHe.return_spectra(params[1])
  
  # Helium fraction   
  tmpflux = Hespec*params[3] + Hspec*(1-params[3])

  # return flux
  flux[:]=tmpflux


# add vvacx as a possible XSPEC model
xspec.AllModels.addPyMod(xspecvvacx, vvacxInfo, 'add')


# Here, actually declare this as the model you want to use
# presumably, most of the time this would not be a stand alone model
# i.e. something like xspec.Model('tbabs(powerlaw+xspecvvacx)')
xspec.Model('xspecvvacx')

# ...then proceed with loading data, fitting, etc.



